﻿using ProjectContainerShip.DrawningObjects;
using ProjectContainerShip.Entities;
using ProjectContainerShip;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TrackBar;

namespace ProjectContainerShip
{
    /// <summary>
    /// Форма создания объекта
    /// </summary>
    public partial class FormContainerConfig : Form
    {
        /// <summary>
        /// Переменная-выбранный корабль
        /// </summary>
        DrawningShip? _ship = null;
        /// <summary>
        /// Событие
        /// </summary>
        private event Action<DrawningShip> EventAddShip;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormContainerConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += panelColor_MouseDown;
            panelPurple.MouseDown += panelColor_MouseDown;
            panelGray.MouseDown += panelColor_MouseDown;
            panelGreen.MouseDown += panelColor_MouseDown;
            panelRed.MouseDown += panelColor_MouseDown;
            panelWhite.MouseDown += panelColor_MouseDown;
            panelYellow.MouseDown += panelColor_MouseDown;
            panelBlue.MouseDown += panelColor_MouseDown;
            // TODO buttonCancel.Click with lambda
            buttonCancel.Click += (sender, e) => Close();
        }
        /// <summary>
        /// Отрисовать контейнеровоз
        /// </summary>
        private void DrawShip()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _ship?.SetPosition(5, 5);
            _ship?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawningShip> ev)
        {
            if (EventAddShip == null)
            {
                EventAddShip = new Action<DrawningShip>(ev);
            }
            else
            {
                EventAddShip += ev;
            }
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _ship = new DrawningShip((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _ship = new DrawningContainerShip((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxContainer.Checked,
                   checkBoxCrane.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawShip();
        }
        /// <summary>
        /// Отправляем цвет с панели
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void panelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_ship != null)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    _ship.EntityShip.BodyColor = (Color)e.Data.GetData(typeof(Color));
                }
                DrawShip();
            }
        }
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (_ship != null && _ship.EntityShip is EntityContainerShip entityustabat)
            {
                labelDopColor.AllowDrop = true;
            }
            else
                labelDopColor.AllowDrop = false;
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_ship != null && _ship.EntityShip is EntityContainerShip entityContainerShip)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    entityContainerShip.AdditionalColor = (Color)e.Data.GetData(typeof(Color));
                }
                DrawShip();
            }
        }
        /// <summary>
        /// Добавление корабля
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddShip?.Invoke(_ship);
            Close();
        }
    }
}
